@@ -195,9 +195,10 @@ static PyObject *PylibMC_Client_get(PylibMC_Client *self, PyObject *arg) {
         PyObject *r = _PylibMC_parse_memcached_value(mc_val, val_size, flags);
         free(mc_val);
         return r;
-    }
-
-    if (error == MEMCACHED_NOTFOUND) {
+    } else if (error == MEMCACHED_SUCCESS) {
+        /* This happens for empty values, and so we fake an empty string. */
+        return PyString_FromStringAndSize("", 0);
+    } else if (error == MEMCACHED_NOTFOUND) {
         /* Since python-memcache returns None when the key doesn't exist,
          * so shall we. */
         Py_RETURN_NONE;
@@ -742,6 +743,98 @@ error:
     return NULL;
 }
 
+static PyObject *PylibMC_Client_get_stats(PylibMC_Client *self, PyObject *args) {
+    memcached_stat_st *stats;
+    memcached_return rc;
+    char *mc_args;
+    PyObject *retval;
+    Py_ssize_t nservers, serveridx;
+    memcached_server_st *servers;
+
+    mc_args = NULL;
+    if (!PyArg_ParseTuple(args, "|s", &mc_args))
+        return NULL;
+
+    stats = memcached_stat(self->mc, mc_args, &rc);
+    if (rc != MEMCACHED_SUCCESS)
+        return PylibMC_ErrFromMemcached(self, "get_stats", rc);
+
+    nservers = (Py_ssize_t)memcached_server_count(self->mc);
+    servers = memcached_server_list(self->mc);
+    retval = PyList_New(nservers);
+
+    /** retval contents:
+     * [('<addr, 127.0.0.1:11211> (<num, 1>),
+     *   {stat: stat, stat: stat}),
+     *  (str, dict),
+     *  (str, dict)]
+     */
+
+    for (serveridx = 0; serveridx < nservers; serveridx++) {
+        memcached_server_st *server;
+        memcached_stat_st *stat;
+        memcached_return rc;
+        PyObject *desc, *val;
+        char **stat_keys = NULL;
+        char **curr_key;
+
+        server = servers + serveridx;
+        stat = stats + serveridx;
+
+        val = PyDict_New();
+        if (val == NULL)
+            goto error;
+
+        stat_keys = memcached_stat_get_keys(self->mc, stat, &rc);
+        if (rc != MEMCACHED_SUCCESS)
+            goto loop_error;
+
+        for (curr_key = stat_keys; *curr_key; curr_key++) {
+            PyObject *curr_value;
+            char *mc_val;
+            memcached_return get_val_rc;
+            int fail;
+
+            mc_val = memcached_stat_get_value(self->mc, stat, *curr_key,
+                                              &get_val_rc);
+            if (get_val_rc != MEMCACHED_SUCCESS) {
+                PylibMC_ErrFromMemcached(self, "get_stats val", get_val_rc);
+                goto loop_error;
+            }
+
+            curr_value = PyString_FromString(mc_val);
+            free(mc_val);
+            if (curr_value == NULL)
+                goto loop_error;
+
+            fail = PyDict_SetItemString(val, *curr_key, curr_value);
+            Py_DECREF(curr_value);
+            if (fail)
+                goto loop_error;
+        }
+
+        free(stat_keys);
+
+        desc = PyString_FromFormat("%s:%d (%u)",
+                server->hostname, server->port,
+                (unsigned int)serveridx);
+        PyList_SET_ITEM(retval, serveridx, Py_BuildValue("NN", desc, val));
+        continue;
+loop_error:
+        free(stat_keys);
+        Py_DECREF(val);
+        goto error;
+    }
+
+    free(stats);
+
+    return retval;
+error:
+    Py_DECREF(retval);
+    free(stats);
+    return NULL;
+}
+
 static PyObject *PylibMC_Client_flush_all(PylibMC_Client *self,
         PyObject *args, PyObject *kwds) {
     memcached_return rc;
@@ -797,8 +890,18 @@ static PyObject *PylibMC_ErrFromMemcached(PylibMC_Client *self, const char *what
         PyErr_Format(PyExc_RuntimeError, "error == 0? %s:%d",
                      __FILE__, __LINE__);
     } else { 
-        PyErr_Format(PylibMCExc_MemcachedError, "error %d from %s: %s",
-                error, what, memcached_strerror(self->mc, error));
+        PylibMC_McErr *err;
+        PyObject *exc = (PyObject *)PylibMCExc_MemcachedError;
+
+        for (err = PylibMCExc_mc_errs; err->name != NULL; err++) {
+            if (err->rc == error) {
+                exc = err->exc;
+                break;
+            }
+        }
+
+        PyErr_Format(exc, "error %d from %s: %s", error, what,
+                     memcached_strerror(self->mc, error));
     }
     return NULL;
 }
@@ -883,6 +986,7 @@ static PyMethodDef PylibMC_functions[] = {
 PyMODINIT_FUNC init_pylibmc(void) {
     PyObject *module;
     PylibMC_Behavior *b;
+    PylibMC_McErr *err;
     char name[128];
 
     if (PyType_Ready(&PylibMC_ClientType) < 0) {
@@ -913,10 +1017,20 @@ Oh, and: plankton.\n");
         return;
     }
 
+    PyModule_AddStringConstant(module, "__version__", PYLIBMC_VERSION);
+
     PylibMCExc_MemcachedError = PyErr_NewException(
             "_pylibmc.MemcachedError", NULL, NULL);
+    PyModule_AddObject(module, "MemcachedError",
+                       (PyObject *)PylibMCExc_MemcachedError);
 
-    PyModule_AddStringConstant(module, "__version__", PYLIBMC_VERSION);
+    for (err = PylibMCExc_mc_errs; err->name != NULL; err++) {
+        char excnam[64];
+        strncpy(excnam, "_pylibmc.", 64);
+        strncat(excnam, err->name, 64);
+        err->exc = PyErr_NewException(excnam, PylibMCExc_MemcachedError, NULL);
+        PyModule_AddObject(module, err->name, (PyObject *)err->exc);
+    }
 
     Py_INCREF(&PylibMC_ClientType);
     PyModule_AddObject(module, "client", (PyObject *)&PylibMC_ClientType);
