@@ -14,6 +14,12 @@ True
 >>> c.get("test_key")
 >>>
 
+We should handle empty values just nicely.
+>>> c.set("foo", "")
+True
+>>> c.get("foo")
+''
+
 Now this section is because most other implementations ignore zero-keys.
 >>> c.get("")
 >>> c.set("", "hi")
@@ -86,6 +92,19 @@ Traceback (most recent call last):
   ...
 TypeError: key must be an instance of str
 
+Getting stats is fun!
+>>> for (svr, stats) in c.get_stats():
+...     print svr
+...     ks = stats.keys()
+...     while ks:
+...         cks, ks = ks[:6], ks[6:]
+...         print ", ".join(cks)
+localhost:11211 (0)
+pid, total_items, uptime, version, limit_maxbytes, rusage_user
+bytes_read, rusage_system, cmd_get, curr_connections, threads, total_connections
+cmd_set, curr_items, get_misses, evictions, bytes, connection_structures
+bytes_written, time, pointer_size, get_hits
+
 Also test some flush all.
 >>> c.set("hi", "guys")
 True
@@ -128,6 +147,16 @@ True
 True
 >>> del c2
 
+Per-error exceptions
+>>> c.incr("test")
+Traceback (most recent call last):
+  ...
+NotFound: error 16 from memcached_increment: NOT FOUND
+>>> c.incr(chr(0))
+Traceback (most recent call last):
+  ...
+ProtocolError: error 8 from memcached_increment: PROTOCOL ERROR
+
 Behaviors.
 >>> c.set_behaviors({"tcp_nodelay": True, "hash": 6})
 >>> list(sorted((k, v) for (k, v) in c.get_behaviors().items()
